# Copyright 2021-2024 Kolena Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Assets are additional files linked to the [`TestSample`][kolena.workflow.TestSample],
[`GroundTruth`][kolena.workflow.GroundTruth], or [`Inference`][kolena.workflow.Inference] objects for your workflow.
Assets can be visualized in the Kolena Studio when exploring your test cases or model results.

The following asset types are available:

- [`ImageAsset`][kolena.workflow.asset.ImageAsset]
- [`PlainTextAsset`][kolena.workflow.asset.PlainTextAsset]
- [`BinaryAsset`][kolena.workflow.asset.BinaryAsset]
- [`PointCloudAsset`][kolena.workflow.asset.PointCloudAsset]
- [`VideoAsset`][kolena.workflow.asset.VideoAsset]

"""
from kolena.asset import Asset
from kolena.asset import BaseVideoAsset
from kolena.asset import BinaryAsset
from kolena.asset import ImageAsset
from kolena.asset import PlainTextAsset
from kolena.asset import PointCloudAsset
from kolena.asset import VideoAsset

__all__ = ["Asset", "ImageAsset", "PlainTextAsset", "BinaryAsset", "PointCloudAsset", "BaseVideoAsset", "VideoAsset"]



# Copyright 2021-2024 Kolena Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
from base64 import b64decode
from base64 import b64encode
from enum import Enum
from io import BytesIO
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Type
from typing import TypeVar
from typing import Union

import dacite
import numpy as np
import pandas as pd


def serialize_embedding_vector(embedding_vector: np.ndarray) -> str:
    memfile = BytesIO()
    np.save(memfile, embedding_vector, allow_pickle=False)
    memfile.seek(0)
    embedding_b64_bytes = b64encode(memfile.read())
    return embedding_b64_bytes.decode("utf-8")


def deserialize_embedding_vector(b64blob: str) -> np.ndarray:
    embedding_bytes = b64decode(b64blob)
    memfile = BytesIO()
    memfile.write(embedding_bytes)
    memfile.seek(0)
    return np.load(memfile)


def as_float64_array(maybe_arr: Optional[List[Union[float, int]]]) -> Optional[np.ndarray]:
    return np.array(maybe_arr).astype(np.float64) if maybe_arr is not None else None


def as_serialized_json(maybe_json: Optional[Union[np.ndarray, Any]]) -> Optional[str]:
    if isinstance(maybe_json, np.ndarray):
        maybe_json = maybe_json.tolist()
    return json.dumps(maybe_json, sort_keys=True) if maybe_json is not None else None


def as_deserialized_json(maybe_json_string: Optional[str]) -> Optional[Any]:
    return json.loads(maybe_json_string) if maybe_json_string is not None else None


def with_serialized_columns(df: pd.DataFrame, object_columns: List[str]) -> pd.DataFrame:
    df_serializable = df.copy()
    for col in object_columns:
        df_serializable[col] = df_serializable[col].apply(as_serialized_json)
    return df_serializable


T = TypeVar("T")
_CONFIG = dacite.Config(cast=[Enum, tuple], check_types=False)


def from_dict(data_class: Type[T], data: Dict[str, Any]) -> T:
    return dacite.from_dict(data_class=data_class, data=data, config=_CONFIG)



# Copyright 2021-2024 Kolena Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from kolena._experimental.object_detection.dataset import upload_object_detection_results


upload_instance_segmentation_results = upload_object_detection_results
"""
Compute metrics and upload results of the model for the dataset.

Dataframe `df` should include a `locator` column that would match to that of corresponding datapoint. Column
:inference in the Dataframe `df` should be a list of scored [`Polygons`][kolena.annotation.Polygon].

:param dataset_name: Dataset name.
:param model_name: Model name.
:param df: Dataframe for model results.
:param ground_truths_field: Field name in datapoint with ground truth polygons, defaulting to `"ground_truths"`.
:param raw_inferences_field: Column in model result DataFrame with raw inference polygons,
defaulting to `"raw_inferences"`.
:param iou_threshold: The [IoU ↗](../../metrics/iou.md) threshold, defaulting to `0.5`.
:param threshold_strategy: The confidence threshold strategy. It can either be a fixed confidence threshold such
    as `0.5` or `0.75`, or `"F1-Optimal"` to find the threshold maximizing F1 score..
:param min_confidence_score: The minimum confidence score to consider for the evaluation. This is usually set to
    reduce noise by excluding inferences with low confidence score.
:return:
"""
